pub struct BareField<T>(pub T);Expand description
A ConfigField wrapper implementation with no metadata.
Used to implement on foreign types that do not implement ConfigField directly.
Tuple Fields
0: TTrait Implementations
Sourceimpl<T> ConfigField for BareField<T>
impl<T> ConfigField for BareField<T>
Sourcetype SpawnHandle = Entity
type SpawnHandle = Entity
Remembers where the config data are stored in the world after spawning.
Sourcetype Reader<'a> = &'a T
type Reader<'a> = &'a T
The type returned when reading the config data from the world. Read more
Sourcetype ReadQueryData = Option<&'static ScalarData<BareField<T>>>
type ReadQueryData = Option<&'static ScalarData<BareField<T>>>
The minimal components required to read the typed config fields under this field. Read more
Sourcetype Metadata = BareMetadata
type Metadata = BareMetadata
Type-specific metadata specified by the referrer. Read more
Sourcetype Changed = FieldGeneration
type Changed = FieldGeneration
Type returned by
ConfigField::changed. Read moreSourcetype ChangedQueryData = ()
type ChangedQueryData = ()
The minimal components required to compute whether the config data has changed. Read more
Sourcefn read_world<'a, 's>(
query: impl QueryLike<Item = <<Self::ReadQueryData as QueryData>::ReadOnly as QueryData>::Item<'a, 's>>,
spawn_handle: &Entity,
) -> Self::Reader<'a>
fn read_world<'a, 's>( query: impl QueryLike<Item = <<Self::ReadQueryData as QueryData>::ReadOnly as QueryData>::Item<'a, 's>>, spawn_handle: &Entity, ) -> Self::Reader<'a>
Reads config data for user consumption from a query of config data entities.
Sourcefn changed<'a, 's>(
query: impl QueryLike<Item = (&'a ConfigNode, <<Self::ChangedQueryData as QueryData>::ReadOnly as QueryData>::Item<'a, 's>)>,
spawn_handle: &Entity,
) -> Self::Changed
fn changed<'a, 's>( query: impl QueryLike<Item = (&'a ConfigNode, <<Self::ChangedQueryData as QueryData>::ReadOnly as QueryData>::Item<'a, 's>)>, spawn_handle: &Entity, ) -> Self::Changed
Computes an equivalence class that represents whether the config data has changed. Read more